#ifndef lint
static char *rcsid = "$Header: /scratch/fock/cvsroot/xd/xdgraph/src/styCvtManager.c,v 2.10 1996/07/25 09:14:52 richter Rel $";
static char *copyright = "Copyright (c) 1995 The XD Group, Free University of Berlin. All rights reserved.";
#endif
/*
 *======================================================================
 *
 * Module: styManager
 *
 * $Log: styCvtManager.c,v $
 * Revision 2.10  1996/07/25 09:14:52  richter
 * new numbers because CV (emacs package) does not unterstand branches
 *
 * Revision 2.9  1996/07/03  14:33:09  richter
 * *** empty log message ***
 *
 * Revision 2.9.1.1  1996/05/19  16:03:43  richter
 * new routine styParseArgs
 *
 * Revision 2.9  1996/04/09  15:54:28  richter
 * first ci for xdgraph (formerly xd2d)
 *
 * Revision 2.0  1995/05/12  11:54:16  richter
 * First semi-public beta release
 *
 *
 */

/*
 *----------------------------------------------------------------------
 * include files
 *----------------------------------------------------------------------
 */
#include "conf.h"               /* global configuration                 */
#include "xdgraph.h"		/* type definitions and array sizes     */
#include "strProto.h"
#include "styProto.h"
#include "pltProto.h"
#ifdef VMS
#include "../../libxd/src/strcasecmp.h"
#endif

/*
 *----------------------------------------------------------------------
 * prototypes with internal linkage
 *----------------------------------------------------------------------
 */
static void TrimYUV(float y, float *u, float *v, float a, float b);

/*
 *----------------------------------------------------------------------
 * variables with internal linkage
 * see also below for variables from the icol program
 *----------------------------------------------------------------------
 */
static ColorT *table = NULL;
static int tablesize = -1;

static struct cmapS {
    int r, g, b, a;
} cmap[2][256];


/*static StyleT style = {
    {NULL, UNKNOWNMODEL, {0., 0., 0.}},
    {0., 0, NULL},
    {0., NULL, NULL}
};
*/

/*
 *----------------------------------------------------------------------
 * functions with external and internal linkage
 *----------------------------------------------------------------------
 */


void
styLoadColorname(char *name)
/*
 * Load table of color names.  This table is constructed from the
 * X11 Windows System database (rgb.txt).  A perl script to convert
 * the format can be found as 'mkrgb.pl'
 */
{
    FILE *f;
    int i, n, irgb[3];
    char col[32];
#ifdef WIN32
    void xdMessageBox(char *string);
#endif

    /*
     * 1. pass:
     * Just count the entries
     */
    f = fopen( name, "r" );
    if (f==NULL) {
#ifdef WIN32
        xdMessageBox("Could not load color table - is environment variable XD_DATADIR set?");
#else
        perror("Could not load color table");
        perror(name);
#endif
        exit(1);
    }
    n = 0;
    while (fscanf(f, "%d %d %d %32s", &irgb[0], &irgb[1], &irgb[2], col)!=EOF)
      n++;

    /*
     * Allocate a sufficiently large color name table (see below)
     */
    tablesize = n;
    if (table) ckfree((char *) table);
    table = (ColorT *) ckalloc((n+1) * sizeof(ColorT));

    /*
     * 2. pass:
     * Fill the color name table
     */
    n = 0;
    rewind(f);
    while (fscanf(f, "%d %d %d %32s", &irgb[0], &irgb[1], &irgb[2], col)!=EOF){
	table[n].name = strDupl(col);
	for (i=0; i<3; ++i) table[n].val[i] = irgb[i] / 255.;
	n++;
    }
    fclose(f);

    /*
     * Add one entry as a default color
     */
    table[n].name = strDupl("unmatched");
    for (i=0; i<3; ++i) table[n].val[i] = 0.;
    return;
}

void
styCvtColorname(ColorT *col)
/*
 * Convert a color name to RGB values
 * TODO: The lookup is primitiv.  At least a binary search should
 * be used.
 */
{
    int i, k;

    for (k=0; k<tablesize; ++k) {
	if (strcasecmp(col->name,table[k].name) == 0) break;
    }

    col->model = RGBMODEL;
    for (i=0; i<3; ++i) {
	col->val[i] = table[k].val[i];
    }
    return;
}

int
styLoadColormap(int id,
		char *file)
{
    FILE *f;
    int i;
    f = fopen(file, "r");
    for (i=0; i<256; ++i) {
	fscanf(f, "%d %d %d %d",
	       &cmap[id][i].r, &cmap[id][i].g, &cmap[id][i].b, &cmap[id][i].a);
    }
    fclose(f);
    return 1;
}

                
int
styCvtindex2rgb(int id,
                int icol,
                float *r,
                float *g,
                float *b)
{
    if (id < 0 || id > 1 || icol < 0 || icol > 255 || cmap[id][icol].a <= 0)
        return 0;
    *r = cmap[id][icol].r/255.;
    *g = cmap[id][icol].g/255.;
    *b = cmap[id][icol].b/255.;
    return 1;
}

/*
 * The following routines to convert between RGB, HSV and YUV color space
 * were taken from the program 'icol' of the GLV package.
 * Only the names are changed to protect the innocent.
 */

/* Copyright (C) 1992  AHPCRC, Univeristy of Minnesota
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program in a file named 'Copying'; if not, write to
 * the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139.
 */

/* Author:
 *	Ken Chin-Purcell (ken@ahpcrc.umn.edu)
 *	Army High Performance Computing Research Center (AHPCRC)
 *	Univeristy of Minnesota
 *
 * Header: /usr/people/ken/gvl/icol/RCS/rgb.c,v 2.1 92/10/19 16:59:34 ken Exp
 *
 * Log:	rgb.c,v
 * Revision 2.1  92/10/19  16:59:34  ken
 * *** empty log message ***
 * 
 * Revision 1.1  92/10/19  16:58:13  ken
 * Initial revision
 * 
 */
/* The following are needed to get around specifying floating point
 * constants for Ultrix cc (i.e. 1.0f not allowed),  Yuck.
 * For speed, we don't want to inline double constants.
 */
static float    F_ZERO  = 0;
static float    F_ONE   = 1;
static float    F_TWO   = 2;
static float    F_FOUR  = 4;
static float    F_SIX   = 6;

/* Return largest of 3 values
 */
float max3(float a, float b, float c)
{    
    if (a > b)
	if (a > c)	return a;
	else		return c;
    else
	if (b > c)	return b;
	else		return c;
}


/* Return smallest of 3 values
 */
float min3(float a, float b, float c)
{
    if (a < b)
	if (a < c)	return a;
	else		return c;
    else
	if (b < c)	return b;
	else		return c;
}


/* Convert coordinates in RGB colour
 * space to coordinates in HSV colour space
 *    In:   r, g, b each in [0,1]
 *    Out:  h in [0, 1), s in (0, 1], and v in [0, 1]
 */
void
styCvtrgb2hsv(float r,  float g,  float b,
	      float *h, float *s, float *v)
{
    float	max = max3(r, g, b);
    float	min = min3(r, g, b);
    float	delta = max - min;

    *v = max;
    
    if (max != F_ZERO  &&  delta != F_ZERO) {

	*s = delta / max;
	if (r == max)
	    *h =          (g - b) / delta;	/* between yellow and magenta */
	else if (g == max)
	    *h = F_TWO  + (b - r) / delta;	/* between cyan and yellow */
	else
	    *h = F_FOUR + (r - g) / delta;	/* between magenta and cyan */
	
	*h /= F_SIX;
	while (*h < F_ZERO)
	    *h += F_ONE;

    } else {
	*s = F_ZERO;
	*h = F_ZERO;
    }
}


/* Convert coordinates in HSV colour space to
 * coordinates in RGB colour space
 *    In:  h in [0, 1), s in (0, 1], and v in [0, 1]
 *    Out: r, g, b each in [0,1]
 */
void
styCvthsv2rgb(float h,  float s,  float v,
	      float *r, float *g, float *b)
{
    float	f, p, q, t;
    
    if (s == F_ZERO) {
	*r = v;
	*g = v;
	*b = v;
	
    } else {
	while (h < F_ZERO)	    h += F_ONE;
	while (h >= F_ONE)	    h -= F_ONE;

	h *= F_SIX;
	f = h - (int) h;    /* fractional part of hue */
	
	p = v * (F_ONE - s);
	q = v * (F_ONE - (s * f));
	t = v * (F_ONE - (s * (F_ONE - f)));
	
	switch ((int) h) {
	  case 0: *r = v; *g = t; *b = p; break;
	  case 1: *r = q; *g = v; *b = p; break;
	  case 2: *r = p; *g = v; *b = t; break;
	  case 3: *r = p; *g = q; *b = v; break;
	  case 4: *r = t; *g = p; *b = v; break;
	  case 5: *r = v; *g = p; *b = q; break;
	}
    }
}


/* YUV color space is a linear rearrangement of RGB.
 * Y is luminance, the black & white channel.
 */

static float Myuv2rgb[3][3] = {
    { 1.00000000,  0.00000000,  1.40200007 },
    { 1.00000000, -0.34413627, -0.71413624 },
    { 1.00000000,  1.77199996,  0.00000000 },
};

static float Mrgb2yuv[3][3] = {
    {  0.29900000,  0.58700000,  0.11400000 },
    { -0.16873589, -0.33126411,  0.50000000 },
    {  0.50000000, -0.41868759, -0.08131241 },
};


/* Rescale U and V so that they are [0,1] outside of rgb.c
 */
static float uBase  = -0.50;
static float vBase  = -0.50;


static void
TrimYUV(float y, float *u, float *v, float a, float b)
{
    /* Scale u and v, keeping y constant, so that color
     * falls within the RGB color unit cube.
     * To derive: start with the RGB = Myuv2rgb * YUV equations,
     *            fix Y, limit  0 <= RGB <= 1.
     *            Pick a new UV along a line from the old UV to UV = (0,0).
     *            UV = (0,0) is always a valid color (grey).
     */
    float	au  = a * *u;
    float	ybv = -y - b * *v;
    
    if (au < ybv) {
	if (*u == F_ZERO) {
	    *v = -y / b;
	} else {
	    float	vu = *v / *u;
	    *u = -y / (a + b * vu);
	    *v = vu * *u;
	}

    } else if (au > F_ONE + ybv) {
	if (*u == F_ZERO) {
	    *v = (F_ONE - y) / b;
	} else {
	    float	vu = *v / *u;
	    *u = (F_ONE - y) / (a + b * vu);
	    *v = vu * *u;
	}
    }
}


void
styCvtyuv2rgb(float y,  float u,  float v,
	float *r, float *g, float *b)
{
    u += uBase;
    v += vBase;
    
    TrimYUV(y, &u, &v, Myuv2rgb[0][1], Myuv2rgb[0][2]);
    TrimYUV(y, &u, &v, Myuv2rgb[1][1], Myuv2rgb[1][2]);
    TrimYUV(y, &u, &v, Myuv2rgb[2][1], Myuv2rgb[2][2]);

    *r = y + Myuv2rgb[0][1] * u + Myuv2rgb[0][2] * v;
    *g = y + Myuv2rgb[1][1] * u + Myuv2rgb[1][2] * v;
    *b = y + Myuv2rgb[2][1] * u + Myuv2rgb[2][2] * v;
}


void
styCvtrgb2yuv(float r,  float g,  float b,
	      float *y, float *u, float *v)
{
    *y = Mrgb2yuv[0][0] * r + Mrgb2yuv[0][1] * g + Mrgb2yuv[0][2] * b;
    *u = Mrgb2yuv[1][0] * r + Mrgb2yuv[1][1] * g + Mrgb2yuv[1][2] * b;
    *v = Mrgb2yuv[2][0] * r + Mrgb2yuv[2][1] * g + Mrgb2yuv[2][2] * b;
    
    *u -= uBase;
    *v -= vBase;
}
