#ifndef lint
static char *rcsid = "$Header: /scratch/fock/cvsroot/xd/xdgraph/src/main.c,v 2.13 1999/06/03 12:41:50 richter Exp $";
static char *copyright = "Copyright (c) 1995 The XD Group, Free University of Berlin. All rights reserved.";
#endif


/*
 *======================================================================
 *
 * Module: main
 *
 * $Log: main.c,v $
 * Revision 2.13  1999/06/03 12:41:50  richter
 * remove two unused variables
 *
 * Revision 2.12  1998/02/09 15:25:44  richter
 * removed all options except -d
 * an optional file is used as an initial tcl script
 *
 * Revision 2.11  1996/11/05  16:01:02  richter
 * default driver is gt if GT_OUTPUT is defined
 *
 * Revision 2.10  1996/07/25  09:14:52  richter
 * new numbers because CV (emacs package) does not unterstand branches
 *
 * Revision 2.9.1.2  1996/07/03  10:04:52  richter
 * new command aliases
 * return result from pltInit
 *
 * Revision 2.9.1.1  1996/05/19  16:03:09  richter
 * new command xydiag
 * removed slice command
 *
 * Revision 2.9  1996/04/09  15:54:28  richter
 * first ci for xdgraph (formerly xd2d)
 *
 * Revision 2.0  1995/05/12  11:54:15  richter
 * First semi-public beta release
 *
 *
 */

/*
 *----------------------------------------------------------------------
 * include files
 *----------------------------------------------------------------------
 */
#include "conf.h"		/* global configuration                 */
#include "xdgraph.h"		/* type definitions and array sizes     */
#include "xydProto.h"
#include "ctrProto.h"
#include "heiProto.h"
#include "isoProto.h"
#include "molProto.h"
#include "pltProto.h"
#include "relProto.h"
#include "pthProto.h"
#include "setProto.h"
#include "strProto.h"
#include "vieProto.h"
#include "../../libxd/src/getopt.h"
extern int SleepCmd(ClientData dummy, Tcl_Interp *interp,
		    int argc, char **argv);
extern int ExecuteVariableCmd(ClientData dummy, Tcl_Interp *interp,
			      int argc, char **argv);
extern int MessageCmd(ClientData dummy, Tcl_Interp *interp,
		      int argc, char **argv);

/*
 *----------------------------------------------------------------------
 * prototypes with internal linkage
 *----------------------------------------------------------------------
 */
static void usage( char *msg );

/*
 *----------------------------------------------------------------------
 * initialize variables with external linkage
 *----------------------------------------------------------------------
 */
Tcl_Interp *interp  = NULL;		/* the interpreter 		*/
char       buf[PRINT_BUFFER_SIZE];
/*
 * this is a little hack, as for VMS tclmain isn't part of
 * libtcl any more (at least not in our distribution)
 */
#ifdef VMS
char *tcl_RcFileName = NULL;
#endif

/*
 *----------------------------------------------------------------------
 * variables with internal linkage
 *----------------------------------------------------------------------
 */
struct commands {
    char        *name;
    Tcl_CmdProc *cmdProc;
};

static struct commands addcmd[] = {
    { "dataset", DatasetCmd },
    { "dset", DatasetCmd },
    { "driver", pltDriverCmd },
    { "contour", ContourCmd },
    { "xydiag", XydiagCmd },
    { "xydiagram", XydiagCmd },
    { "height", HeightCmd },
    { "heightfield", HeightCmd },
    { "iso", IsosurfaceCmd },
    { "isosurface", IsosurfaceCmd },
    { "molecule", MoleculeCmd },
    { "path", PathCmd },
    { "relief", ReliefCmd },
    { "view", ViewCmd },
    { "sleep", SleepCmd },
    /*    { "executeVariable", ExecuteVariableCmd },*/
    { "printmsg", MessageCmd },
    { (char *) NULL, (int (*)()) NULL }
};

char *__progname;

/*
 *----------------------------------------------------------------------
 *
 * main
 *
 *----------------------------------------------------------------------
 */
int
main(int argc,
     char **argv)
{
    /*
     * local variables
     */
#ifdef WIN32
    void SETXDVAR();
    SETXDVAR();
/*    
    tcldir = getenv("XD_TCLDIR");
    fprintf(stderr, "%s\n",tcldir);
    datdir = getenv("XD_DATADIR");
    fprintf(stderr, "%s\n",datdir); 
    tcllib = getenv("TCL_LIBRARY");
    fprintf(stderr, "%s\n",tcllib);   
*/               
#endif 

    Tcl_DString initdrv;	/* initial XD commands, constructed from
				 * the command line. */
    char *tcldir, *datdir,*tcllib;
    static char initxd[] =
        "if [file exists $xd_library/xd.tcl] {\n\
            source $xd_library/xd.tcl\n\
        } else {\n\
            set msg \"Can't find $xd_library/xd.tcl; perhaps you \"\n\
            append msg \"need to set the XD_TCLDIR \"\n\
            append msg \"environment variable?\"\n\
            error $msg\n\
        }";

    struct commands *cmdPtr;
    int code, tty;

    int c;
    extern char *optarg;
    extern int optind;
#ifdef WIN32
    void xdMessageBox(char *string);
#endif
#ifdef GT_OUTPUT
    static char defaultDriver[] = "gt";
#else
    static char defaultDriver[] = "tk";
#endif
    char *driver = defaultDriver;
    char *file = NULL;



    /*
     * store program name
     */
    __progname = argv[0];

    /*
     * parse command line
     */
    while ((c = getopt(argc, argv, "d:")) != -1) {
	switch (c) {
	case 'd':
	    driver = optarg;
	    break;
	}
    }
    if (optind < argc) {
	file = argv[optind];
    }

    /*
     * initialize command interpreter
     * add memory debuging commands
     * 
     */
    
    interp = Tcl_CreateInterp();
    Tcl_InitMemory(interp);

    /*
     * Make command-line arguments available in the Tcl variables "argc"
     * and "argv".
    args = Tcl_Merge(argc-1, argv+1);
    Tcl_SetVar(interp, "argv", args, TCL_GLOBAL_ONLY);
    ckfree(args);
    sprintf(buf, "%d", argc-1);
    Tcl_SetVar(interp, "argc", buf, TCL_GLOBAL_ONLY);
    Tcl_SetVar(interp, "argv0", argv[0], TCL_GLOBAL_ONLY);
    */

    /*
     * Set the "tcl_interactive" variable.
     */
    tty = isatty(0);
    Tcl_SetVar(interp, "tcl_interactive", tty ? "1" : "0", TCL_GLOBAL_ONLY);
    if (Tcl_Init(interp) == TCL_ERROR) {
#ifdef WIN32
        xdMessageBox(interp->result);
#else
	fprintf(stderr, "Tcl_Init failed: %s\n", interp->result);
#endif
    }

    /*
     * add new commands
     */
    for (cmdPtr = addcmd; cmdPtr->name != NULL; cmdPtr++) {
	Tcl_CreateCommand( interp, cmdPtr->name, cmdPtr->cmdProc,
			  (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL );
    };

    /*
     * set global variables
     */
    tcldir = getenv("XD_TCLDIR");
    if (tcldir == NULL) {
	tcldir = getenv("XD_LIBRARY");
	if (tcldir == NULL) {
	    tcldir = XD_TCLDIR;
	}
    }
    Tcl_SetVar(interp, "xd_tcldir",  tcldir, TCL_GLOBAL_ONLY);
    Tcl_SetVar(interp, "xd_library", tcldir, TCL_GLOBAL_ONLY);

    datdir = getenv("XD_DATADIR");
    if (datdir == NULL) {
	datdir = XD_DATADIR;
    }
    Tcl_SetVar(interp, "xd_datadir", datdir, TCL_GLOBAL_ONLY);

    /*
     * load initial XD script
     */
    code = Tcl_Eval(interp, initxd);
    if (code != TCL_OK) {
#ifdef WIN32
        xdMessageBox(interp->result);
#else
	fprintf(stderr, "%s\n", interp->result);
#endif
	exit(1);
    }

/*    Tcl_InitHistory(interp);*/
    /*
     * construct initial XD commands
     */
    Tcl_DStringInit(&initdrv);
    if (file) {
	Tcl_DStringAppendElement(&initdrv, "source");
	Tcl_DStringAppendElement(&initdrv, file);
    }

    /*
     * initialize driver, 'initdrv' may be executed there
     * NOTE: This routine does not return.
     */
    code = pltInit(driver, Tcl_DStringValue(&initdrv));
    if (code != TCL_OK) {
#ifdef WIN32
        xdMessageBox(interp->result);
#else
        fprintf(stderr, "%s\n", interp->result);
#endif
    }
    return code;
    /*    return 0; /* */
}

static void
usage( char *msg )
{
    fprintf(stderr,"%s: %s\n", __progname, msg);
    fprintf(stderr,"usage: contour [options] [tcl-script]\n");
    fprintf(stderr,"\t-d driver\n");
    exit(1);
}

int
SleepCmd(ClientData dummy,		/* Not used. */
	 Tcl_Interp *interp,		/* Current interpreter. */
	 int argc,			/* Number of arguments. */
	 char **argv)			/* Argument strings. */
{
    int seconds;

    if (argc != 2) {
        Tcl_AppendResult(interp, "wrong # args:  should be \"",
			 argv[0], " time\"", (char *) NULL);
        return TCL_ERROR;
    }
  
    if (Tcl_GetInt(interp, argv[1], &seconds) != TCL_OK) {
	return TCL_ERROR;
    }

    if (seconds < 0) {
        Tcl_AppendResult(interp, "bad time", (char *) NULL);
        return TCL_ERROR;
    }

    sleep(seconds);
    return TCL_OK;
}

int
ExecuteVariableCmd(ClientData dummy,		/* Not used. */
		   Tcl_Interp *interp,		/* Current interpreter. */
		   int argc,			/* Number of arguments. */
		   char **argv)	                /* Argument strings. */
{
    int result;
    char *cmd;

    if (argc != 2) {
        Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
                " arg ?arg ...?\"", (char *) NULL);
        return TCL_ERROR;
    }
    cmd = Tcl_GetVar(interp, argv[1], TCL_GLOBAL_ONLY);
    if (cmd) {
        printf("recording %s\n", cmd);
	result = Tcl_RecordAndEval(interp, cmd, 0);
	if (result == TCL_ERROR) {
	    char msg[60];
	    sprintf(msg, "\n    (\"eval\" variable '%s' line %d)",
		    argv[1], interp->errorLine);
	    Tcl_AddErrorInfo(interp, msg);
	}
    } else {
	result = TCL_ERROR;
    }
    return result;
}

int
MessageCmd(ClientData dummy,		/* Not used. */
	   Tcl_Interp *interp,		/* Current interpreter. */
	   int argc,			/* Number of arguments. */
	   char **argv)	                /* Argument strings. */
{
    static char *deflev = "inf";
    char *level;

    if (argc > 1) {
        if (argc < 2) {
            level = deflev;
        } else {
            if (strEQ(argv[1], "inf", 3) ||
                strEQ(NULL, "err", 3) ||
                strEQ(NULL, "cmd", 3)) {
                level = argv[1];
            } else {
                level = deflev;
            }
        }
	return pltMessage(level, argv[2]);
    }
    return TCL_OK;
}
